==> mcss-0.0.35.tar.gz: mcss-0.0.35/mcss/MulticompartmentGillespieQ2.cpp <==
// multicompartment gillespie direct method with queue class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespieQ2.h"

#include <limits>
#include <ecsb/ecsb.h>
#include <gsl/gsl_sf_log.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespieQ2::MulticompartmentGillespieQ2(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie Queue 2", seed)
{
	waiting_times = NULL;
}

// destructor
MulticompartmentGillespieQ2::~MulticompartmentGillespieQ2()
{
	if(waiting_times)
		bhpq_free(waiting_times);
}

// initialise algorithm
int MulticompartmentGillespieQ2::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// allocate memory for hash of global waiting times
	if(waiting_times)
		bhpq_free(waiting_times);
	waiting_times = bhpq_init(num_compartments);

	// calculate tau and mu for each compartment
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// add tau to map of compartment waiting times
		if(tautmp != 0.0) {
			qe.key1 = tautmp + tau0global;
			qe.key2 = (int) i;
			bhpq_insert(&qe, waiting_times);
		}
	}

	// finish if no more reactions to execute
	if(!bhpq_peek_head(&qe, waiting_times)) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// get compartment with smallest waiting time
	tau0global = qe.key1;
	i0 = (unsigned long) qe.key2;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie algorithm with queue
int MulticompartmentGillespieQ2::execute(void)
{
	register unsigned long it;
	register double tautmp;
	register Compartment *compartment;

	// update simulation time
	setTime(tau0global);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);

	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);

	// add new waiting time to map of compartment waiting times
	tautmp = compartment->getTimeToNextReaction();
	if(tautmp != 0.0) {
		qe.key1 = tautmp + tau0global;
		qe.key2 = (int) i0;
		bhpq_replace_head(&qe, waiting_times);
	} else
		// remove old waiting time from map of compartment waiting times
		bhpq_delete_head(waiting_times);

	// update target compartment if channel
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// add new waiting time to map of compartment waiting times
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction();
		if(tautmp != 0.0) {
			qe.key1 = tautmp + tau0global;
			qe.key2 = (int) it;
			bhpq_insert(&qe, waiting_times);
		}
	}

	// finish if no more reactions to execute
	if(!bhpq_peek_head(&qe, waiting_times)) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// get compartment with smallest waiting time
	tau0global = qe.key1;
	i0 = (unsigned long) qe.key2;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespieQ2::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);

	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = randomNumberPos();
	tau0 = -1.0 * gsl_sf_log(r1) / a0;

	// select next reaction
	r2 = randomNumberPos();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}

	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}

==> mcss-1.0.1.tar.gz: mcss-1.0.1/mcss/MulticompartmentGillespieQ2.cpp <==
// multicompartment gillespie direct method with queue class definitions
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "MulticompartmentGillespieQ2.h"

#include <limits>
#include <ecsb/ecsb.h>
#include <gsl/gsl_sf_log.h>

using namespace Simulation;
using namespace ECSB;

// constructor
MulticompartmentGillespieQ2::MulticompartmentGillespieQ2(const unsigned long seed) \
		: SimulationAlgorithm("Multicompartment Gillespie Queue 2", seed)
{
	waiting_times = NULL;
}

// destructor
MulticompartmentGillespieQ2::~MulticompartmentGillespieQ2()
{
	if(waiting_times)
		bhpq_free(waiting_times);
}

// initialise algorithm
int MulticompartmentGillespieQ2::preprocess(void)
{
	register unsigned long i;
	register double tautmp;
	Compartment *compartment;

	// allocate memory for hash of global waiting times
	if(waiting_times)
		bhpq_free(waiting_times);
	waiting_times = bhpq_init(num_compartments);

	// calculate tau and mu for each compartment
	for(i = 0; i < num_compartments; i++) {
		compartment = psystem->getCompartment(i);
		psystem->execute(i);
		tautmp = compartment->getTimeToNextReaction();
		// add tau to map of compartment waiting times
		if(tautmp != 0.0) {
			qe.key1 = tautmp + tau0global;
			qe.key2 = (int) i;
			bhpq_insert(&qe, waiting_times);
		}
	}

	// finish if no more reactions to execute
	if(!bhpq_peek_head(&qe, waiting_times)) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// get compartment with smallest waiting time
	tau0global = qe.key1;
	i0 = (unsigned long) qe.key2;
	compartment = psystem->getCompartment(i0);
	tau0 = compartment->getTimeToNextReaction();
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for all compartments
// using multicompartment gillespie algorithm with queue
int MulticompartmentGillespieQ2::execute(void)
{
	register unsigned long it;
	register double tautmp;
	register Compartment *compartment;

	// update simulation time
	setTime(tau0global);

	// perform reaction in selected compartment
	compartment = psystem->getCompartment(i0);
	psystem->executeRule(rs0, mu0, i0);

	// run gillespie to recalculate next reaction in selected compartment
	psystem->execute(i0);

	// add new waiting time to map of compartment waiting times
	tautmp = compartment->getTimeToNextReaction();
	if(tautmp != 0.0) {
		qe.key1 = tautmp + tau0global;
		qe.key2 = (int) i0;
		bhpq_replace_head(&qe, waiting_times);
	} else
		// remove old waiting time from map of compartment waiting times
		bhpq_delete_head(waiting_times);

	// update target compartment if channel
	if(compartment->isChannel(rs0, mu0)) {
		it = compartment->getRuleTarget(rs0, mu0);
		// run gillespie to recalculate next reaction in target compartment
		psystem->execute(it);
		// add new waiting time to map of compartment waiting times
		compartment = psystem->getCompartment(it);
		tautmp = compartment->getTimeToNextReaction();
		if(tautmp != 0.0) {
			qe.key1 = tautmp + tau0global;
			qe.key2 = (int) it;
			bhpq_insert(&qe, waiting_times);
		}
	}

	// handle events
	if(events_defined) {
		for(it = 0; it < num_event_compartments; it++) {
			compartment = psystem->getCompartment(event_compartment[it]);
			if(compartment->handleEvents(tau0global)) {
				// event executed
				// run gillespie to recalculate next reaction in event compartment
				psystem->execute(event_compartment[it]);
				// add new waiting time to map of compartment waiting times
				tautmp = compartment->getTimeToNextReaction();
				if(tautmp != 0.0) {
					qe.key1 = tautmp + tau0global;
					qe.key2 = (int) event_compartment[it];
					bhpq_insert(&qe, waiting_times);
				}
			}
		}
	}

	// finish if no more reactions to execute
	if(!bhpq_peek_head(&qe, waiting_times)) {
		tau0 = mu0 = i0 = rs0 = 0;
		return 0;
	}

	// get compartment with smallest waiting time
	tau0global = qe.key1;
	i0 = (unsigned long) qe.key2;
	compartment = psystem->getCompartment(i0);
	tau0 = tau0global - time;
	mu0 = compartment->getNextReaction();
	rs0 = compartment->getNextRuleSet();

	return 1;
}

// perform one iteration of algorithm for one compartment
// using gillespie direct method
void MulticompartmentGillespieQ2::execute(unsigned long c)
{
	register unsigned long i, mu0, rs0;
	register double a0, tau0, r1, r2, am;
	register Compartment *compartment;

	// don't run for compartments with no rules
//	if(!compartment->getNumRules())
//		return;

	compartment = psystem->getCompartment(c);

	// calculate propensities
	a0 = 0.0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(i = 0; i < compartment->getNumRules(rs0); i++)
			a0 += calculatePropensity(rs0, i, c);

	// finish if zero level
	if(a0 == 0.0) {
		compartment->setTimeToNextReaction(0.0);
		compartment->setNextReaction(0);
		compartment->setNextRuleSet(0);
		return;
	}

	// generate time to next reaction
	r1 = rng->randomNumberPos();
	tau0 = -1.0 * gsl_sf_log(r1) / a0;

	// select next reaction
	r2 = rng->randomNumberPos();
	am = r2 * a0;
	for(rs0 = 0; rs0 < compartment->getNumRuleSets(); rs0++)
		for(mu0 = 0; mu0 < compartment->getNumRules(rs0); mu0++) {
			am -= compartment->getPropensity(rs0, mu0);
			if(am <= 0.0)
				goto done;
		}

	done:
		compartment->setTimeToNextReaction(tau0);
		compartment->setNextReaction(mu0);
		compartment->setNextRuleSet(rs0);
}
